package com.plexworlds.l3.llm

import io.github.amithkoujalgi.ollama4j.core.OllamaAPI
import io.github.amithkoujalgi.ollama4j.core.utils.Options
import io.github.amithkoujalgi.ollama4j.core.utils.OptionsBuilder
import java.net.http.HttpTimeoutException

object Ollama : LLM {

    private var model = "codellama:7b-code"


    override fun call(prefix: String, suffix: String): String {
        if (!isPingOk) {
            return "<No connection to Ollama API>"
        }

        for (i in 0..<RETRY_COUNT) {
            val suggestion = try {
                OllamaAPI(HOST).apply {
                    setRequestTimeoutSeconds(4)
                }.generate(model, "<PRE> $prefix <SUF>$suffix <MID>", options).response.let {
                    if (it.endsWith(END)) it.substring(0, it.length - END.length).trim(' ', '\t', '\n') else it
                }
            } catch (e: HttpTimeoutException) {
                continue
            }
            if (suggestion.isNotBlank()) {
                return suggestion
            }
        }

        return "<Ollama API timeout>"
    }


    override fun changeModel(model: String) {
        Ollama.model = model
    }


    private val options: Options by lazy {
        OptionsBuilder()
            .setTemperature(0.4f)
            .build()
    }


    private val isPingOk: Boolean
        get() {
            return try {
                OllamaAPI(HOST).ping()
            } catch (e: Exception) {
                false
            }
        }


    private const val HOST = "http://localhost:44000/"


    private const val END = "<EOT>"

    private const val RETRY_COUNT = 4
}
